from django.db import models
from django.urls import reverse
from netbox.models import NetBoxModel
from dcim.models import Site


class Sitemap(NetBoxModel):
    name = models.CharField(
        max_length=100
    )
    site_groups = models.ManyToManyField(
        to='dcim.SiteGroup',
        related_name='sitemaps',
        blank=True
    )
    sites = models.ManyToManyField(
        to='dcim.Site',
        related_name='sitemaps',
        blank=True
    )
    regions = models.ManyToManyField(
        to='dcim.Region',
        related_name='sitemaps',
        blank=True
    )
    comments = models.TextField(
        blank=True
    )

    class Meta:
        ordering = ("name",)

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse("plugins:netbox_sitemap:sitemap", args=[self.pk])

    def get_markers(self):
        markers = []
        site_ids = []
        # requesting site_ids from sites
        for site in self.sites.all():
            if site.longitude != None and site.latitude != None and site.id not in site_ids:
                site_ids.append(site.id)
        # requesting site_ids from site-groups
        for group in self.site_groups.all():
            for site in Site.objects.filter(group=group):
                if site.longitude != None and site.latitude != None and site.id not in site_ids:
                    site_ids.append(site.id)
        # requesting site_ids from regions
        for region in self.regions.all():
            for site in Site.objects.filter(region=region):
                if site.longitude != None and site.latitude != None and site.id not in site_ids:
                    site_ids.append(site.id)
        # populating markers with site information
        for site_id in site_ids:
            site = Site.objects.get(id=site_id)
            markers.append({
                'type': 'Feature',
                'properties': {
                    'url': site.get_absolute_url(),
                    'name': site.name,
                    'iconSize': [40, 40]
                },
                'geometry': {
                    'type': 'Point',
                    'coordinates': [
                        site.longitude,
                        site.latitude
                    ]
                }
            })
        return markers


from django import forms
from django.template.loader import render_to_string
from extras.dashboard.utils import register_widget
from extras.dashboard.widgets import DashboardWidget, WidgetConfigForm
from .models import Sitemap

def get_sitemap_choices():
    return [
        (sitemap.id, sitemap.name)
        for sitemap in Sitemap.objects.all()
    ]

@register_widget
class SitemapWidget(DashboardWidget):
    default_title = 'Sitemap'
    description = 'Display your sites on a map.'
    template_name = 'netbox_sitemap/sitemapwidget.html'

    class ConfigForm(WidgetConfigForm):
        sitemap = forms.ChoiceField(
            choices=get_sitemap_choices()
        )

    def render(self, request):
        sitemap_id = self.config.get('sitemap')
        return render_to_string(self.template_name, {'object_id': sitemap_id})


from netbox.views import generic
from utilities.views import ViewTab, register_model_view

# Plugin specific imports
from .models import Sitemap
from .filtersets import SitemapFilterSet
from .forms import SitemapFilterForm, SitemapForm
from .tables import SitemapTable


@register_model_view(model=Sitemap)
class SitemapView(generic.ObjectView):
    queryset = Sitemap.objects.all()
    template_name = 'netbox_sitemap/sitemap.html'


@register_model_view(model=Sitemap, name='map', path='map')
class SitemapMapView(generic.ObjectView):
    queryset = Sitemap.objects.all()
    template_name = 'netbox_sitemap/sitemapmap.html'
    tab = ViewTab(
        label='Map',
    )


class SitemapListView(generic.ObjectListView):
    queryset = Sitemap.objects.all()
    table = SitemapTable
    filterset = SitemapFilterSet
    filterset_form = SitemapFilterForm


class SitemapEditView(generic.ObjectEditView):
    queryset = Sitemap.objects.all()
    form = SitemapForm


class SitemapDeleteView(generic.ObjectDeleteView):
    queryset = Sitemap.objects.all()



# Generated by Django 5.2.7 on 2025-11-26 20:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dcim', '0215_rackreservation_status'),
        ('netbox_sitemap', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='sitemap',
            name='regions',
            field=models.ManyToManyField(blank=True, related_name='sitemaps', to='dcim.region'),
        ),
    ]